@@ -23,7 +23,6 @@ from oslo_log import log as logging
 from oslo_utils import encodeutils
 from oslo_utils import secretutils as secutils
 from oslo_utils import strutils
-import six
 import webob.dec
 import webob.exc
 
@@ -71,6 +70,7 @@ class MetadataRequestHandler(wsgi.Application):
         try:
             data = base.get_metadata_by_address(address)
         except exception.NotFound:
+            LOG.exception('Failed to get metadata for IP %s', address)
             return None
 
         if CONF.api.metadata_cache_expiration > 0:
@@ -141,8 +141,6 @@ class MetadataRequestHandler(wsgi.Application):
 
     def _handle_remote_ip_request(self, req):
         remote_address = req.remote_addr
-        if CONF.api.use_forwarded_for:
-            remote_address = req.headers.get('X-Forwarded-For', remote_address)
 
         try:
             meta_data = self.get_metadata_by_remote_address(remote_address)
@@ -151,8 +149,7 @@ class MetadataRequestHandler(wsgi.Application):
                           remote_address)
             msg = _('An unknown error has occurred. '
                     'Please try your request again.')
-            raise webob.exc.HTTPInternalServerError(
-                                               explanation=six.text_type(msg))
+            raise webob.exc.HTTPInternalServerError(explanation=str(msg))
 
         if meta_data is None:
             LOG.error('Failed to get metadata for IP %s: no metadata',
@@ -174,9 +171,9 @@ class MetadataRequestHandler(wsgi.Application):
             msg = _('X-Instance-ID-Signature header is missing from request.')
         elif tenant_id is None:
             msg = _('X-Tenant-ID header is missing from request.')
-        elif not isinstance(instance_id, six.string_types):
+        elif not isinstance(instance_id, str):
             msg = _('Multiple X-Instance-ID headers found within request.')
-        elif not isinstance(tenant_id, six.string_types):
+        elif not isinstance(tenant_id, str):
             msg = _('Multiple X-Tenant-ID headers found within request.')
         else:
             msg = None
@@ -262,7 +259,7 @@ class MetadataRequestHandler(wsgi.Application):
 
         # instance_data is unicode-encoded, while cache_utils doesn't like
         # that. Therefore we convert to str
-        if isinstance(instance_id, six.text_type):
+        if isinstance(instance_id, str):
             instance_id = instance_id.encode('utf-8')
         return instance_id, tenant_id
 
@@ -329,8 +326,7 @@ class MetadataRequestHandler(wsgi.Application):
                           instance_id)
             msg = _('An unknown error has occurred. '
                     'Please try your request again.')
-            raise webob.exc.HTTPInternalServerError(
-                                               explanation=six.text_type(msg))
+            raise webob.exc.HTTPInternalServerError(explanation=str(msg))
 
         if meta_data is None:
             LOG.error('Failed to get metadata for instance id: %s',
