@@ -47,7 +47,7 @@ TEST_HYPERS = [
          vcpus_used=2,
          memory_mb_used=5 * 1024,
          local_gb_used=125,
-         hypervisor_type="xen",
+         hypervisor_type="qemu",
          hypervisor_version=3,
          hypervisor_hostname="hyper1",
          free_ram_mb=5 * 1024,
@@ -67,7 +67,7 @@ TEST_HYPERS = [
          vcpus_used=2,
          memory_mb_used=5 * 1024,
          local_gb_used=125,
-         hypervisor_type="xen",
+         hypervisor_type="qemu",
          hypervisor_version=3,
          hypervisor_hostname="hyper2",
          free_ram_mb=5 * 1024,
@@ -76,7 +76,8 @@ TEST_HYPERS = [
          running_vms=2,
          cpu_info=CPU_INFO,
          disk_available_least=100,
-         host_ip=netaddr.IPAddress('2.2.2.2'))]
+         host_ip=netaddr.IPAddress('2.2.2.2'),
+         stats={'uptime': 'fake uptime'})]
 
 
 TEST_SERVICES = [
@@ -203,6 +204,11 @@ class HypervisorsTestV21(test.NoDBTestCase):
     del DETAIL_HYPERS_DICTS[1]['host']
     del DETAIL_HYPERS_DICTS[0]['uuid']
     del DETAIL_HYPERS_DICTS[1]['uuid']
+    # Remove stats since it's not exposed in the API response, but preserve
+    # uptime for v2.88+ tests which expect it
+    for hyper_dict in DETAIL_HYPERS_DICTS:
+        if 'stats' in hyper_dict:
+            del hyper_dict['stats']
     DETAIL_HYPERS_DICTS[0].update({'state': 'up',
                            'status': 'enabled',
                            'service': dict(id=1, host='compute1',
@@ -852,7 +858,7 @@ class HypervisorsTestV233(HypervisorsTestV228):
                 'free_ram_mb': 5120,
                 'host_ip': netaddr.IPAddress('2.2.2.2'),
                 'hypervisor_hostname': 'hyper2',
-                'hypervisor_type': 'xen',
+                'hypervisor_type': 'qemu',
                 'hypervisor_version': 3,
                 'id': 2,
                 'local_gb': 250,
@@ -907,7 +913,7 @@ class HypervisorsTestV233(HypervisorsTestV228):
                 'free_ram_mb': 5120,
                 'host_ip': netaddr.IPAddress('2.2.2.2'),
                 'hypervisor_hostname': 'hyper2',
-                'hypervisor_type': 'xen',
+                'hypervisor_type': 'qemu',
                 'hypervisor_version': 3,
                 'id': 2,
                 'local_gb': 250,
@@ -954,7 +960,7 @@ class HypervisorsTestV233(HypervisorsTestV228):
                 'free_ram_mb': 5120,
                 'host_ip': netaddr.IPAddress('2.2.2.2'),
                 'hypervisor_hostname': 'hyper2',
-                'hypervisor_type': 'xen',
+                'hypervisor_type': 'qemu',
                 'hypervisor_version': 3,
                 'id': 2,
                 'local_gb': 250,
@@ -1450,6 +1456,21 @@ class HypervisorsTestV288(HypervisorsTestV275):
         # cpu_info is no longer included in the response, so skip this test
         pass
 
+    def test_show_with_uptime_provided_by_compute_node(self):
+        req = self._get_request(use_admin_context=True)
+        result = self.controller.show(req, self.TEST_HYPERS_OBJ[1].uuid)
+        expected_dict = copy.deepcopy(self.DETAIL_HYPERS_DICTS[1])
+        self.assertEqual({'hypervisor': expected_dict}, result)
+        self.controller.host_api.get_host_uptime.assert_not_called()
+
+    def test_detail_list_uptime(self):
+        _ = self._test_servers_with_no_servers(self.controller.detail)
+        # we have simulated that compute 2 is upgraded to store the uptime
+        # in the stats  so we expect 1 call to get the result via RPC
+        # for compute1
+        self.controller.host_api.get_host_uptime.assert_called_with(
+            mock.ANY, "compute1")
+
     def test_uptime(self):
         req = self._get_request(True)
         self.assertRaises(
