@@ -13,8 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import mock
 from neutronclient.common import exceptions as n_exc
+from unittest import mock
+
 from oslo_config import cfg
 from oslo_serialization import jsonutils
 from oslo_utils import encodeutils
@@ -76,7 +77,7 @@ def security_group_rule_template(**kwargs):
     rule.setdefault('ip_protocol', 'tcp')
     rule.setdefault('from_port', 22)
     rule.setdefault('to_port', 22)
-    rule.setdefault('parent_group_id', 2)
+    rule.setdefault('parent_group_id', uuids.parent_group_id)
     return rule
 
 
@@ -453,7 +454,8 @@ class TestSecurityGroupsV21(test.TestCase):
     def test_create_security_group(self):
         sg = security_group_request_template()
 
-        res_dict = self.controller.create(self.req, {'security_group': sg})
+        res_dict = self.controller.create(
+            self.req, body={'security_group': sg})
         self.assertEqual(res_dict['security_group']['name'], 'test')
         self.assertEqual(res_dict['security_group']['description'],
                          'test-description')
@@ -462,31 +464,36 @@ class TestSecurityGroupsV21(test.TestCase):
         sg = security_group_request_template()
         del sg['name']
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, self.req,
-                          {'security_group': sg})
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create, self.req,
+            body={'security_group': sg})
 
     def test_create_security_group_with_no_body(self):
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, self.req, None)
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create, self.req, body=None)
 
     def test_create_security_group_with_no_security_group(self):
         body = {'no-securityGroup': None}
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, self.req, body)
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create, self.req, body=body)
 
     def test_create_security_group_above_255_characters_name(self):
         sg = security_group_request_template(name='1234567890' * 26)
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group': sg})
+        self.assertRaises(
+            exception.ValidationError, self.controller.create,
+            self.req, body={'security_group': sg})
 
     def test_create_security_group_above_255_characters_description(self):
         sg = security_group_request_template(description='1234567890' * 26)
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group': sg})
+        self.assertRaises(
+            nova.exception.ValidationError, self.controller.create,
+            self.req, body={'security_group': sg})
 
     def test_get_security_group_list(self):
         self._create_sg_template().get('security_group')
@@ -505,14 +512,16 @@ class TestSecurityGroupsV21(test.TestCase):
 
     def test_get_security_group_list_missing_group_id_rule(self):
         groups = []
-        rule1 = security_group_rule_template(cidr='10.2.3.124/24',
-                                             parent_group_id=1,
-                                             group_id={}, id=88,
-                                             protocol='TCP')
-        rule2 = security_group_rule_template(cidr='10.2.3.125/24',
-                                             parent_group_id=1,
-                                             id=99, protocol=88,
-                                             group_id='HAS_BEEN_DELETED')
+        rule1 = security_group_rule_template(
+            cidr='10.2.3.124/24',
+            parent_group_id=uuids.parent_group_id,
+            group_id={}, id=88,
+            protocol='TCP')
+        rule2 = security_group_rule_template(
+            cidr='10.2.3.125/24',
+            parent_group_id=uuids.parent_group_id,
+            id=99, protocol=88,
+            group_id='HAS_BEEN_DELETED')
         sg = security_group_template(id=1,
                                      name='test',
                                      description='test-desc',
@@ -524,7 +533,8 @@ class TestSecurityGroupsV21(test.TestCase):
         # passed in. For example:
         #  "cidr": "0.0.0.0/0" ->"ip_range": {"cidr": "0.0.0.0/0"}
         expected_rule = security_group_rule_template(
-            ip_range={'cidr': '10.2.3.124/24'}, parent_group_id=1,
+            ip_range={'cidr': '10.2.3.124/24'},
+            parent_group_id=uuids.parent_group_id,
             group={}, id=88, ip_protocol='TCP')
         expected = security_group_template(id=1,
                                      name='test',
@@ -602,7 +612,7 @@ class TestSecurityGroupsV21(test.TestCase):
         sg = security_group_template()
 
         self.assertRaises(webob.exc.HTTPBadRequest, self.controller.update,
-                          self.req, '1', {'security_group': sg})
+                          self.req, '1', body={'security_group': sg})
 
     def test_delete_security_group_by_id(self):
         sg = self._create_sg_template().get('security_group')
@@ -710,26 +720,30 @@ class TestSecurityGroupsV21(test.TestCase):
     def test_associate_by_non_existing_security_group_name(self):
         body = dict(addSecurityGroup=dict(name='non-existing'))
 
-        self.assertRaises(webob.exc.HTTPNotFound,
-                          self.manager._addSecurityGroup, self.req, '1', body)
+        self.assertRaises(
+            webob.exc.HTTPNotFound,
+            self.manager._addSecurityGroup, self.req, '1', body=body)
 
     def test_associate_without_body(self):
         body = dict(addSecurityGroup=None)
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.manager._addSecurityGroup, self.req, '1', body)
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            self.manager._addSecurityGroup, self.req, '1', body=body)
 
     def test_associate_no_security_group_name(self):
         body = dict(addSecurityGroup=dict())
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.manager._addSecurityGroup, self.req, '1', body)
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            self.manager._addSecurityGroup, self.req, '1', body=body)
 
     def test_associate_security_group_name_with_whitespaces(self):
         body = dict(addSecurityGroup=dict(name="   "))
 
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.manager._addSecurityGroup, self.req, '1', body)
+        self.assertRaises(
+            webob.exc.HTTPBadRequest,
+            self.manager._addSecurityGroup, self.req, '1', body=body)
 
     def test_associate_non_existing_instance(self):
         body = dict(addSecurityGroup=dict(name="test"))
@@ -738,7 +752,7 @@ class TestSecurityGroupsV21(test.TestCase):
                             instance_id='1')):
             self.assertRaises(webob.exc.HTTPNotFound,
                               self.manager._addSecurityGroup,
-                              self.req, '1', body)
+                              self.req, '1', body=body)
 
     def test_associate_duplicate_names(self):
         sg1 = self._create_security_group(name='sg1',
@@ -757,7 +771,7 @@ class TestSecurityGroupsV21(test.TestCase):
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
         self.assertRaises(webob.exc.HTTPConflict,
                           self.manager._addSecurityGroup,
-                          req, UUID_SERVER, body)
+                          req, UUID_SERVER, body=body)
 
     def test_associate_port_security_enabled_true(self):
         sg = self._create_sg_template().get('security_group')
@@ -772,7 +786,7 @@ class TestSecurityGroupsV21(test.TestCase):
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/servers/%s/action' %
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
-        self.manager._addSecurityGroup(req, UUID_SERVER, body)
+        self.manager._addSecurityGroup(req, UUID_SERVER, body=body)
 
     def test_associate_port_security_enabled_false(self):
         self._create_sg_template().get('security_group')
@@ -788,7 +802,7 @@ class TestSecurityGroupsV21(test.TestCase):
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.manager._addSecurityGroup,
-                          req, UUID_SERVER, body)
+                          req, UUID_SERVER, body=body)
 
     def test_associate_deferred_ip_port(self):
         sg = self._create_sg_template().get('security_group')
@@ -803,7 +817,7 @@ class TestSecurityGroupsV21(test.TestCase):
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/servers/%s/action' %
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
-        self.manager._addSecurityGroup(req, UUID_SERVER, body)
+        self.manager._addSecurityGroup(req, UUID_SERVER, body=body)
 
     def test_associate(self):
         sg = self._create_sg_template().get('security_group')
@@ -817,7 +831,7 @@ class TestSecurityGroupsV21(test.TestCase):
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/servers/%s/action' %
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
-        self.manager._addSecurityGroup(req, UUID_SERVER, body)
+        self.manager._addSecurityGroup(req, UUID_SERVER, body=body)
 
     def test_disassociate_by_non_existing_security_group_name(self):
         body = dict(removeSecurityGroup=dict(name='non-existing'))
@@ -827,28 +841,28 @@ class TestSecurityGroupsV21(test.TestCase):
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
         self.assertRaises(webob.exc.HTTPNotFound,
                           self.manager._removeSecurityGroup,
-                          req, UUID_SERVER, body)
+                          req, UUID_SERVER, body=body)
 
     def test_disassociate_without_body(self):
         body = dict(removeSecurityGroup=None)
 
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.manager._removeSecurityGroup, self.req,
-                          '1', body)
+                          '1', body=body)
 
     def test_disassociate_no_security_group_name(self):
         body = dict(removeSecurityGroup=dict())
 
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.manager._removeSecurityGroup, self.req,
-                          '1', body)
+                          '1', body=body)
 
     def test_disassociate_security_group_name_with_whitespaces(self):
         body = dict(removeSecurityGroup=dict(name="   "))
 
         self.assertRaises(webob.exc.HTTPBadRequest,
                           self.manager._removeSecurityGroup, self.req,
-                          '1', body)
+                          '1', body=body)
 
     def test_disassociate_non_existing_instance(self):
         body = dict(removeSecurityGroup=dict(name="test"))
@@ -857,7 +871,7 @@ class TestSecurityGroupsV21(test.TestCase):
                             instance_id='1')):
             self.assertRaises(webob.exc.HTTPNotFound,
                               self.manager._removeSecurityGroup,
-                              self.req, '1', body)
+                              self.req, '1', body=body)
 
     def test_disassociate(self):
         sg = self._create_sg_template().get('security_group')
@@ -871,7 +885,7 @@ class TestSecurityGroupsV21(test.TestCase):
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/servers/%s/action' %
                 (fakes.FAKE_PROJECT_ID, UUID_SERVER))
-        self.manager._removeSecurityGroup(req, UUID_SERVER, body)
+        self.manager._removeSecurityGroup(req, UUID_SERVER, body=body)
 
     def test_get_instances_security_groups_bindings(self):
         servers = [{'id': FAKE_UUID1}, {'id': FAKE_UUID2}]
@@ -972,8 +986,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(cidr='10.2.3.124/24',
                                             parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
         self.assertEqual(security_group_rule['parent_group_id'],
@@ -985,8 +999,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(group_id=self.sg1['id'],
                                             parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1003,8 +1017,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
                                              from_port=81, to_port=81,
                                              parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule2})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule2})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1016,8 +1030,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
     def test_create_none_value_from_to_port(self):
         rule = {'parent_group_id': self.sg1['id'],
                 'group_id': self.sg1['id']}
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         self.assertIsNone(security_group_rule['from_port'])
         self.assertIsNone(security_group_rule['to_port'])
@@ -1029,8 +1043,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = {'parent_group_id': self.sg1['id'],
                 'group_id': self.sg1['id'],
                 'ip_protocol': 'ICMP'}
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         self.assertEqual(security_group_rule['ip_protocol'], 'ICMP')
         self.assertEqual(security_group_rule['from_port'], -1)
@@ -1043,8 +1057,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = {'parent_group_id': self.sg1['id'],
                 'group_id': self.sg1['id'],
                 'ip_protocol': 'TCP'}
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         self.assertEqual(security_group_rule['ip_protocol'], 'TCP')
         self.assertEqual(security_group_rule['from_port'], 1)
@@ -1060,8 +1074,9 @@ class TestSecurityGroupRulesV21(test.TestCase):
                 to_port=22,
                 parent_group_id=self.sg2['id'],
                 cidr="10.2.3.124/2433")
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_by_invalid_tcp_port_json(self):
         rule = security_group_rule_template(
@@ -1071,8 +1086,9 @@ class TestSecurityGroupRulesV21(test.TestCase):
                 parent_group_id=self.sg2['id'],
                 cidr="10.2.3.124/24")
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_by_invalid_icmp_port_json(self):
         rule = security_group_rule_template(
@@ -1081,140 +1097,158 @@ class TestSecurityGroupRulesV21(test.TestCase):
                 to_port=256,
                 parent_group_id=self.sg2['id'],
                 cidr="10.2.3.124/24")
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_add_existing_rules_by_cidr(self):
         sg = security_group_template()
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/os-security-groups' % fakes.FAKE_PROJECT_ID)
-        self.controller_sg.create(req, {'security_group': sg})
+        self.controller_sg.create(req, body={'security_group': sg})
         rule = security_group_rule_template(
             cidr='15.0.0.0/8', parent_group_id=self.sg2['id'])
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/os-security-group-rules' % fakes.FAKE_PROJECT_ID)
-        self.controller.create(req, {'security_group_rule': rule})
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          req, {'security_group_rule': rule})
+        self.controller.create(req, body={'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            req, body={'security_group_rule': rule})
 
     def test_create_add_existing_rules_by_group_id(self):
         sg = security_group_template()
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/os-security-groups' % fakes.FAKE_PROJECT_ID)
-        self.controller_sg.create(req, {'security_group': sg})
+        self.controller_sg.create(req, body={'security_group': sg})
         rule = security_group_rule_template(
             group=self.sg1['id'], parent_group_id=self.sg2['id'])
         req = fakes.HTTPRequest.blank(
                 '/v2/%s/os-security-group-rules' % fakes.FAKE_PROJECT_ID)
-        self.controller.create(req, {'security_group_rule': rule})
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          req, {'security_group_rule': rule})
+        self.controller.create(req, body={'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            req, body={'security_group_rule': rule})
 
     def test_create_with_no_body(self):
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, self.req, None)
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create, self.req, body=None)
 
     def test_create_with_no_security_group_rule_in_body(self):
         rules = {'test': 'test'}
-        self.assertRaises(webob.exc.HTTPBadRequest,
-                          self.controller.create, self.req, rules)
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create, self.req, body=rules)
 
     def test_create_with_invalid_parent_group_id(self):
         rule = security_group_rule_template(parent_group_id='invalid')
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            exception.ValidationError,
+            self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_non_existing_parent_group_id(self):
         rule = security_group_rule_template(group_id=None,
                                             parent_group_id=self.invalid_id)
 
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPNotFound, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_non_existing_group_id(self):
-        rule = security_group_rule_template(group_id='invalid',
+        rule = security_group_rule_template(group_id=uuids.doesnotexist,
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPNotFound, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_invalid_protocol(self):
         rule = security_group_rule_template(ip_protocol='invalid-protocol',
                                             cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_no_protocol(self):
         rule = security_group_rule_template(cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
         del rule['ip_protocol']
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_invalid_from_port(self):
         rule = security_group_rule_template(from_port='666666',
                                             cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_invalid_to_port(self):
         rule = security_group_rule_template(to_port='666666',
                                             cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_non_numerical_from_port(self):
         rule = security_group_rule_template(from_port='invalid',
                                             cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_non_numerical_to_port(self):
         rule = security_group_rule_template(to_port='invalid',
                                             cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_no_from_port(self):
         rule = security_group_rule_template(cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
         del rule['from_port']
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_no_to_port(self):
         rule = security_group_rule_template(cidr='10.2.2.0/24',
                                             parent_group_id=self.sg2['id'])
         del rule['to_port']
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_invalid_cidr(self):
         rule = security_group_rule_template(cidr='10.2.2222.0/24',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_no_cidr_group(self):
         rule = security_group_rule_template(parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1227,28 +1261,31 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(group_id='invalid',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            exception.ValidationError, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_empty_group_id(self):
         rule = security_group_rule_template(group_id='',
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            exception.ValidationError, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_nonexist_group_id(self):
         rule = security_group_rule_template(group_id=self.invalid_id,
                                             parent_group_id=self.sg2['id'])
 
-        self.assertRaises(webob.exc.HTTPNotFound, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPNotFound, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def test_create_with_same_group_parent_id_and_group_id(self):
         rule = security_group_rule_template(group_id=self.sg1['id'],
                                             parent_group_id=self.sg1['id'])
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
         self.assertEqual(security_group_rule['parent_group_id'],
@@ -1259,15 +1296,16 @@ class TestSecurityGroupRulesV21(test.TestCase):
     def _test_create_with_no_ports_and_no_group(self, proto):
         rule = {'ip_protocol': proto, 'parent_group_id': self.sg2['id']}
 
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
 
     def _test_create_with_no_ports(self, proto):
         rule = {'ip_protocol': proto, 'parent_group_id': self.sg2['id'],
                  'group_id': self.sg1['id']}
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         expected_rule = {
             'from_port': 1, 'group': {'tenant_id': '123', 'name': 'test'},
@@ -1296,8 +1334,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
             'ip_protocol': proto, 'from_port': from_port, 'to_port': to_port,
             'parent_group_id': self.sg2['id'], 'group_id': self.sg1['id']
         }
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         expected_rule = {
@@ -1331,7 +1369,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
 
         req = fakes.HTTPRequest.blank(
             '/v2/%s/os-security-group-rules' % fakes.FAKE_PROJECT_ID)
-        res_dict = self.controller.create(req, {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            req, body={'security_group_rule': rule})
         security_group_rule = res_dict['security_group_rule']
         req = fakes.HTTPRequest.blank('/v2/%s/os-security-group-rules/%s' % (
                 fakes.FAKE_PROJECT_ID, security_group_rule['id']))
@@ -1349,8 +1388,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(cidr='0.0.0.0/0',
                                             parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1363,8 +1402,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(cidr='::/0',
                                             parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1377,8 +1416,8 @@ class TestSecurityGroupRulesV21(test.TestCase):
         rule = security_group_rule_template(cidr='15.0.0.0/8',
                                             parent_group_id=self.sg2['id'])
 
-        res_dict = self.controller.create(self.req,
-            {'security_group_rule': rule})
+        res_dict = self.controller.create(
+            self.req, body={'security_group_rule': rule})
 
         security_group_rule = res_dict['security_group_rule']
         self.assertNotEqual(security_group_rule['id'], 0)
@@ -1388,9 +1427,12 @@ class TestSecurityGroupRulesV21(test.TestCase):
                          "15.0.0.0/8")
 
     def test_create_rule_cidr_bad_netmask(self):
-        rule = security_group_rule_template(cidr='15.0.0.0/0')
-        self.assertRaises(webob.exc.HTTPBadRequest, self.controller.create,
-                          self.req, {'security_group_rule': rule})
+        rule = security_group_rule_template(cidr='15.0.0.0/0',
+                                            parent_group_id=self.sg2['id'])
+        exc = self.assertRaises(
+            webob.exc.HTTPBadRequest, self.controller.create,
+            self.req, body={'security_group_rule': rule})
+        self.assertIn('Bad prefix for network ', str(exc))
 
 
 UUID1 = '00000000-0000-0000-0000-000000000001'
@@ -1501,7 +1543,7 @@ class SecurityGroupsOutputTest(test.TestCase):
         security_groups = [{'name': 'fake-2-0'}, {'name': 'fake-2-1'}]
         for security_group in security_groups:
             sg = security_group_template(name=security_group['name'])
-            self.controller.create(req, {'security_group': sg})
+            self.controller.create(req, body={'security_group': sg})
 
         server = dict(name='server_test', imageRef=image_uuid, flavorRef=2,
                       security_groups=security_groups)
@@ -1535,7 +1577,7 @@ class SecurityGroupsOutputTest(test.TestCase):
         security_groups = [{'name': 'fake-2-0'}, {'name': 'fake-2-1'}]
         for security_group in security_groups:
             sg = security_group_template(name=security_group['name'])
-            self.controller.create(req, {'security_group': sg})
+            self.controller.create(req, body={'security_group': sg})
         server = dict(name='server_test', imageRef=image_uuid, flavorRef=2,
                       security_groups=security_groups)
 
@@ -1591,9 +1633,9 @@ class TestSecurityGroupsDeprecation(test.NoDBTestCase):
         self.assertRaises(exception.VersionNotFoundForAPIMethod,
             self.controller.index, self.req)
         self.assertRaises(exception.VersionNotFoundForAPIMethod,
-            self.controller.update, self.req, fakes.FAKE_UUID, {})
+            self.controller.update, self.req, fakes.FAKE_UUID, body={})
         self.assertRaises(exception.VersionNotFoundForAPIMethod,
-            self.controller.create, self.req, {})
+            self.controller.create, self.req, body={})
 
 
 class TestSecurityGroupRulesDeprecation(test.NoDBTestCase):
@@ -1605,6 +1647,6 @@ class TestSecurityGroupRulesDeprecation(test.NoDBTestCase):
 
     def test_all_apis_return_not_found(self):
         self.assertRaises(exception.VersionNotFoundForAPIMethod,
-            self.controller.create, self.req, {})
+            self.controller.create, self.req, body={})
         self.assertRaises(exception.VersionNotFoundForAPIMethod,
             self.controller.delete, self.req, fakes.FAKE_UUID)
