@@ -14,7 +14,6 @@
 
 import eventlet
 import os
-import six
 import time
 
 import os_resource_classes as orc
@@ -47,6 +46,10 @@ class ZVMDriver(driver.ComputeDriver):
     """z/VM implementation of ComputeDriver."""
     capabilities = {
         "supports_pcpus": False,
+        "supports_remote_managed_ports": False,
+        "supports_address_space_passthrough": False,
+        "supports_address_space_emulated": False,
+        "supports_stateless_firmware": False,
 
         # Image type support flags
         "supports_image_type_aki": False,
@@ -128,6 +131,7 @@ class ZVMDriver(driver.ComputeDriver):
                                      obj_fields.HVType.ZVM,
                                      obj_fields.VMMode.HVM)],
             'numa_topology': None,
+            'stats': {'uptime': self.get_host_uptime()}
         }
 
         LOG.debug("Getting available resource for %(host)s:%(nodename)s",
@@ -291,7 +295,7 @@ class ZVMDriver(driver.ComputeDriver):
                 raise exception.VirtualInterfaceCreateException()
         except Exception as err:
             with excutils.save_and_reraise_exception():
-                LOG.error("Failed for vif plugging: %s", six.text_type(err),
+                LOG.error("Failed for vif plugging: %s", str(err),
                           instance=instance)
 
     def _import_spawn_image(self, context, image_meta_id, image_os_version):
